@@ -39,7 +39,7 @@ gc.stroke_width 3
 gc.circle 170,220, 173,223
 
 # Add labels
-gc.stroke "#000000ff"       # unset stroke color
+gc.stroke "none"       # unset stroke color
 gc.fill 'black'
 
 # Add end point labels
@@ -50,7 +50,7 @@ gc.text 330,125, "'320,120'"
 # Add control point labels
 gc.text  30, 25, "'20,20'"
 gc.text 180, 25, "'170,20'"
-gc.text 330,225, "'320,320'"
+gc.text 330,225, "'320,220'"
 
 # Add auto control point label
 gc.text 180,225, "'auto ctl point'"
