@@ -1,6 +1,6 @@
-/* $Id: rmdraw.c,v 1.43.2.2.2.1 2007/11/25 19:42:42 rmagick Exp $ */
+/* $Id: rmdraw.c,v 1.64 2008/05/21 22:32:40 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2007 by Timothy P. Hunter
+|                Copyright (C) 2008 by Timothy P. Hunter
 | Name:     rmdraw.c
 | Author:   Tim Hunter
 | Purpose:  Contains Draw class methods.
@@ -28,7 +28,7 @@ Draw_affine_eq(VALUE self, VALUE matrix)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     AffineMatrix_to_AffineMatrix(&draw->info->affine, matrix);
     return self;
@@ -43,7 +43,7 @@ Draw_align_eq(VALUE self, VALUE align)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(align, draw->info->align, AlignType);
     return self;
@@ -58,7 +58,7 @@ Draw_decorate_eq(VALUE self, VALUE decorate)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(decorate, draw->info->decorate, DecorationType);
     return self;
@@ -73,9 +73,9 @@ Draw_density_eq(VALUE self, VALUE density)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->density, STRING_PTR(density));
+    magick_clone_string(&draw->info->density, StringValuePtr(density));
 
     return self;
 }
@@ -89,9 +89,9 @@ Draw_encoding_eq(VALUE self, VALUE encoding)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->encoding, STRING_PTR(encoding));
+    magick_clone_string(&draw->info->encoding, StringValuePtr(encoding));
 
     return self;
 }
@@ -105,12 +105,44 @@ Draw_fill_eq(VALUE self, VALUE fill)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->fill, fill);
     return self;
 }
 
+/*
+    Method:     Draw#fill_pattern=
+    Purpose:    Accept an image as a fill pattern
+    Notes:      See also stroke_pattern=, tile=
+*/
+VALUE
+Draw_fill_pattern_eq(VALUE self, VALUE pattern)
+{
+    Draw *draw;
+    Image *image;
+
+    rb_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+
+    if (draw->info->fill_pattern != NULL)
+    {
+        // Do not trace destruction
+        DestroyImage(draw->info->fill_pattern);
+        draw->info->fill_pattern = NULL;
+    }
+
+    if (!NIL_P(pattern))
+    {
+        pattern = ImageList_cur_image(pattern);
+        image = rm_check_destroyed(pattern);
+        // Do not trace creation
+        draw->info->fill_pattern = rm_clone_image(image);
+    }
+
+    return self;
+}
+
 /*
     Method:     Draw#font=
     Purpose:    font attribute writer
@@ -120,9 +152,9 @@ Draw_font_eq(VALUE self, VALUE font)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->font, STRING_PTR(font));
+    magick_clone_string(&draw->info->font, StringValuePtr(font));
 
     return self;
 }
@@ -136,9 +168,9 @@ Draw_font_family_eq(VALUE self, VALUE family)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
-    magick_clone_string(&draw->info->family, STRING_PTR(family));
+    magick_clone_string(&draw->info->family, StringValuePtr(family));
 
     return self;
 }
@@ -152,7 +184,7 @@ Draw_font_stretch_eq(VALUE self, VALUE stretch)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(stretch, draw->info->stretch, StretchType);
     return self;
@@ -167,7 +199,7 @@ Draw_font_style_eq(VALUE self, VALUE style)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(style, draw->info->style, StyleType);
     return self;
@@ -187,7 +219,7 @@ Draw_font_weight_eq(VALUE self, VALUE weight)
     Draw *draw;
     WeightType w;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
     if (FIXNUM_P(weight))
@@ -250,7 +282,7 @@ Draw_gravity_eq(VALUE self, VALUE grav)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     VALUE_TO_ENUM(grav, draw->info->gravity, GravityType);
 
@@ -266,7 +298,7 @@ Draw_pointsize_eq(VALUE self, VALUE pointsize)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     draw->info->pointsize = NUM2DBL(pointsize);
     return self;
@@ -285,7 +317,7 @@ Draw_rotation_eq(VALUE self, VALUE deg)
     double degrees;
     AffineMatrix affine, current;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
     degrees = NUM2DBL(deg);
@@ -323,12 +355,45 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->stroke, stroke);
     return self;
 }
 
+/*
+    Method:     Draw#stroke_pattern=
+    Purpose:    Accept an image as a stroke pattern
+    Notes:      See also fill_pattern=
+*/
+VALUE
+Draw_stroke_pattern_eq(VALUE self, VALUE pattern)
+{
+    Draw *draw;
+    Image *image;
+
+    rb_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+
+    if (draw->info->stroke_pattern != NULL)
+    {
+        // Do not trace destruction
+        DestroyImage(draw->info->stroke_pattern);
+        draw->info->stroke_pattern = NULL;
+    }
+
+    if (!NIL_P(pattern))
+    {
+        // DestroyDrawInfo destroys the clone
+        pattern = ImageList_cur_image(pattern);
+        image = rm_check_destroyed(pattern);
+        // Do not trace creation
+        draw->info->stroke_pattern = rm_clone_image(image);
+    }
+
+    return self;
+}
+
 /*
     Method:     Draw#stroke_width=
     Purpose:    stroke_width attribute writer
@@ -336,12 +401,12 @@ Draw_stroke_eq(VALUE self, VALUE stroke)
 VALUE
 Draw_stroke_width_eq(VALUE self, VALUE stroke_width)
 {
-  Draw *draw;
+    Draw *draw;
 
-  rm_check_frozen(self);
-  Data_Get_Struct(self, Draw, draw);
-  draw->info->stroke_width = NUM2DBL(stroke_width);
-  return self;
+    rb_check_frozen(self);
+    Data_Get_Struct(self, Draw, draw);
+    draw->info->stroke_width = NUM2DBL(stroke_width);
+    return self;
 }
 
 /*
@@ -353,7 +418,7 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     draw->info->text_antialias = (MagickBooleanType) RTEST(text_antialias);
     return self;
@@ -366,27 +431,7 @@ Draw_text_antialias_eq(VALUE self, VALUE text_antialias)
 VALUE
 Draw_tile_eq(VALUE self, VALUE image)
 {
-    Draw *draw;
-    Image *tile;
-
-    rm_check_frozen(self);
-    Data_Get_Struct(self, Draw, draw);
-
-    if (draw->info->fill_pattern)
-    {
-        (void) DestroyImage(draw->info->fill_pattern);
-        draw->info->fill_pattern = NULL;
-    }
-
-    if (image == Qnil)
-    {
-        return self;
-    }
-
-    Data_Get_Struct(ImageList_cur_image(image), Image, tile);
-    draw->info->fill_pattern = rm_clone_image(tile);
-
-    return self;
+    return Draw_fill_pattern_eq(self, image);
 }
 
 /*
@@ -398,7 +443,7 @@ Draw_undercolor_eq(VALUE self, VALUE undercolor)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
     Color_to_PixelPacket(&draw->info->undercolor, undercolor);
     return self;
@@ -413,13 +458,13 @@ Draw_undercolor_eq(VALUE self, VALUE undercolor)
                 Draw object.
 */
 VALUE Draw_annotate(
-    VALUE self,
-    VALUE image_arg,
-    VALUE width_arg,
-    VALUE height_arg,
-    VALUE x_arg,
-    VALUE y_arg,
-    VALUE text)
+                   VALUE self,
+                   VALUE image_arg,
+                   VALUE width_arg,
+                   VALUE height_arg,
+                   VALUE x_arg,
+                   VALUE y_arg,
+                   VALUE text)
 {
     Draw *draw;
     Image *image;
@@ -433,6 +478,9 @@ VALUE Draw_annotate(
     Data_Get_Struct(self, Draw, draw);
     keep = draw->info->affine;
 
+    image_arg = ImageList_cur_image(image_arg);
+    image = rm_check_frozen(image_arg);
+
     // If we have an optional parm block, run it in self's context,
     // allowing the app a chance to modify the object's attributes
     if (rb_block_given_p())
@@ -440,18 +488,15 @@ VALUE Draw_annotate(
         (void)rb_obj_instance_eval(0, NULL, self);
     }
 
-    rm_check_frozen(ImageList_cur_image(image_arg));
-    Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
-
     // Translate & store in Draw structure
-#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
-    draw->info->text = InterpretImageAttributes(NULL, image, STRING_PTR(text));
+#if defined(HAVE_INTERPRETIMAGEPROPERTIES)
+    draw->info->text = InterpretImageProperties(NULL, image, StringValuePtr(text));
 #else
-    draw->info->text = TranslateText(NULL, image, STRING_PTR(text));
+    draw->info->text = InterpretImageAttributes(NULL, image, StringValuePtr(text));
 #endif
     if (!draw->info->text)
     {
-           rb_raise(rb_eArgError, "no text");
+        rb_raise(rb_eArgError, "no text");
     }
 
     // Create geometry string, copy to Draw structure, overriding
@@ -498,7 +543,7 @@ Draw_clone(VALUE self)
     clone = Draw_dup(self);
     if (OBJ_FROZEN(self))
     {
-        (void)rb_obj_freeze(clone);
+        OBJ_FREEZE(clone);
     }
 
     return clone;
@@ -520,9 +565,9 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     CompositeOperator cop = OverCompositeOp;
     volatile VALUE image;
     Image *comp_img;
-    struct TmpFile_Name *tmpfile;
+    struct TmpFile_Name *tmpfile_name;
     char name[MaxTextExtent];
-                            // Buffer for "image" primitive
+    // Buffer for "image" primitive
     char primitive[MaxTextExtent];
 
     if (argc < 5 || argc > 6)
@@ -530,6 +575,10 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
         rb_raise(rb_eArgError, "wrong number of arguments (%d for 5 or 6)", argc);
     }
 
+    // Retrieve the image to composite
+    image = ImageList_cur_image(argv[4]);
+    (void) rm_check_destroyed(image);
+
     x = NUM2DBL(argv[0]);
     y = NUM2DBL(argv[1]);
     width  = NUM2DBL(argv[2]);
@@ -540,7 +589,7 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     {
         VALUE_TO_ENUM(argv[5], cop, CompositeOperator);
 
-        switch(cop)
+        switch (cop)
         {
             case AddCompositeOp:
                 op = "Add";
@@ -604,9 +653,6 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
 
     Data_Get_Struct(self, Draw, draw);
 
-    // Retrieve the image to composite
-    image = ImageList_cur_image(argv[4]);
-
     // Create a temp copy of the composite image
     Data_Get_Struct(image, Image, comp_img);
     rm_write_temp_image(comp_img, name);
@@ -614,10 +660,10 @@ Draw_composite(int argc, VALUE *argv, VALUE self)
     // Add the temp filename to the filename array.
     // Use Magick storage since we need to keep the list around
     // until destroy_Draw is called.
-    tmpfile = magick_malloc(sizeof(struct TmpFile_Name)+strlen(name));
-    strcpy(tmpfile->name, name);
-    tmpfile->next = draw->tmpfile_ary;
-    draw->tmpfile_ary = tmpfile;
+    tmpfile_name = magick_malloc(sizeof(struct TmpFile_Name)+strlen(name));
+    strcpy(tmpfile_name->name, name);
+    tmpfile_name->next = draw->tmpfile_ary;
+    draw->tmpfile_ary = tmpfile_name;
 
     // Form the drawing primitive
     (void) sprintf(primitive, "image %s %g,%g,%g,%g '%s'", op, x, y, width, height, name);
@@ -640,17 +686,17 @@ Draw_draw(VALUE self, VALUE image_arg)
     Draw *draw;
     Image *image;
 
+    image_arg = ImageList_cur_image(image_arg);
+    image = rm_check_frozen(image_arg);
+
     Data_Get_Struct(self, Draw, draw);
     if (draw->primitives == 0)
     {
         rb_raise(rb_eArgError, "nothing to draw");
     }
 
-    rm_check_frozen(ImageList_cur_image(image_arg));
-    Data_Get_Struct(ImageList_cur_image(image_arg), Image, image);
-
     // Point the DrawInfo structure at the current set of primitives.
-    magick_clone_string(&(draw->info->primitive), STRING_PTR(draw->primitives));
+    magick_clone_string(&(draw->info->primitive), StringValuePtr(draw->primitives));
 
     (void) DrawImage(image, draw->info);
     rm_check_image_exception(image, RetainOnError);
@@ -694,9 +740,9 @@ Draw_dup(VALUE self)
 */
 VALUE
 Draw_get_type_metrics(
-    int argc,
-    VALUE *argv,
-    VALUE self)
+                     int argc,
+                     VALUE *argv,
+                     VALUE self)
 {
     return get_type_metrics(argc, argv, self, GetTypeMetrics);
 }
@@ -704,16 +750,11 @@ Draw_get_type_metrics(
 
 VALUE
 Draw_get_multiline_type_metrics(
-     int argc,
-     VALUE *argv,
-     VALUE self)
+                               int argc,
+                               VALUE *argv,
+                               VALUE self)
 {
-#if defined(HAVE_GETMULTILINETYPEMETRICS)
-     return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
-#else
-     rm_not_implemented();
-     return (VALUE)0;
-#endif
+    return get_type_metrics(argc, argv, self, GetMultilineTypeMetrics);
 }
 
 /*
@@ -781,11 +822,7 @@ Draw_inspect(VALUE self)
     Raises:     ImageMagickError if no memory
 */
 
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 VALUE Draw_alloc(VALUE class)
-#else
-VALUE Draw_new(VALUE class)
-#endif
 {
     Draw *draw;
     volatile VALUE draw_obj;
@@ -794,10 +831,6 @@ VALUE Draw_new(VALUE class)
     memset(draw, 0, sizeof(Draw));
     draw_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw);
 
-#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-    rb_obj_call_init(draw_obj, 0, NULL);
-#endif
-
     return draw_obj;
 }
 
@@ -811,7 +844,7 @@ Draw_primitive(VALUE self, VALUE primitive)
 {
     Draw *draw;
 
-    rm_check_frozen(self);
+    rb_check_frozen(self);
     Data_Get_Struct(self, Draw, draw);
 
     if (draw->primitives == (VALUE)0)
@@ -850,7 +883,7 @@ static void
 destroy_Draw(void *drawptr)
 {
     Draw *draw = (Draw *)drawptr;
-    struct TmpFile_Name *tmpfile;
+    struct TmpFile_Name *tmpfile_name;
 
     if (draw->info)
     {
@@ -861,10 +894,10 @@ destroy_Draw(void *drawptr)
     // Erase any temporary image files.
     while (draw->tmpfile_ary)
     {
-        tmpfile = draw->tmpfile_ary;
+        tmpfile_name = draw->tmpfile_ary;
         draw->tmpfile_ary = draw->tmpfile_ary->next;
-        rm_delete_temp_image(tmpfile->name);
-        magick_free(tmpfile);
+        rm_delete_temp_image(tmpfile_name->name);
+        magick_free(tmpfile_name);
     }
 
     xfree(drawptr);
@@ -877,11 +910,7 @@ destroy_Draw(void *drawptr)
 */
 static VALUE new_DrawOptions(void)
 {
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
     return DrawOptions_initialize(Draw_alloc(Class_DrawOptions));
-#else
-    return DrawOptions_new(Class_DrawOptions);
-#endif
 }
 
 
@@ -892,11 +921,7 @@ static VALUE new_DrawOptions(void)
     Notes:      The DrawOptions class is the same as the Draw class except
                 is has only the attribute writer functions.
 */
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 VALUE DrawOptions_alloc(VALUE class)
-#else
-VALUE DrawOptions_new(VALUE class)
-#endif
 {
     Draw *draw_options;
     volatile VALUE draw_options_obj;
@@ -905,10 +930,6 @@ VALUE DrawOptions_new(VALUE class)
     memset(draw_options, 0, sizeof(Draw));
     draw_options_obj = Data_Wrap_Struct(class, mark_Draw, destroy_Draw, draw_options);
 
-#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-    rb_obj_call_init(draw_options_obj, 0, NULL);
-#endif
-
     return draw_options_obj;
 }
 
@@ -1006,7 +1027,7 @@ Montage_filename_eq(VALUE self, VALUE filename)
     Montage *montage;
 
     Data_Get_Struct(self, Montage, montage);
-    strncpy(montage->info->filename, STRING_PTR(filename), MaxTextExtent-1);
+    strncpy(montage->info->filename, StringValuePtr(filename), MaxTextExtent-1);
     return self;
 }
 
@@ -1034,7 +1055,7 @@ Montage_font_eq(VALUE self, VALUE font)
     Montage *montage;
 
     Data_Get_Struct(self, Montage, montage);
-    magick_clone_string(&montage->info->font, STRING_PTR(font));
+    magick_clone_string(&montage->info->font, StringValuePtr(font));
 
     return self;
 }
@@ -1054,7 +1075,7 @@ Montage_frame_eq(VALUE self, VALUE frame_arg)
 
     Data_Get_Struct(self, Montage, montage);
     frame = rb_funcall(frame_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->frame, STRING_PTR(frame));
+    magick_clone_string(&montage->info->frame, StringValuePtr(frame));
 
     return self;
 }
@@ -1071,7 +1092,7 @@ Montage_geometry_eq(VALUE self, VALUE geometry_arg)
 
     Data_Get_Struct(self, Montage, montage);
     geometry = rb_funcall(geometry_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->geometry, STRING_PTR(geometry));
+    magick_clone_string(&montage->info->geometry, StringValuePtr(geometry));
 
     return self;
 }
@@ -1120,11 +1141,7 @@ Montage_matte_color_eq(VALUE self, VALUE color)
     Purpose:    Create a new Montage object
 */
 VALUE
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 Montage_alloc(VALUE class)
-#else
-Montage_new(VALUE class)
-#endif
 {
     MontageInfo *montage_info;
     Montage *montage;
@@ -1151,10 +1168,6 @@ Montage_new(VALUE class)
     montage->compose = OverCompositeOp;
     montage_obj = Data_Wrap_Struct(class, NULL, destroy_Montage, montage);
 
-#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-    (void) rb_obj_call_init(montage_obj, 0, NULL);
-#endif
-
     return montage_obj;
 }
 
@@ -1165,11 +1178,7 @@ Montage_new(VALUE class)
 
 VALUE rm_montage_new(void)
 {
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
     return Montage_initialize(Montage_alloc(Class_Montage));
-#else
-    return Montage_new(Class_Montage);
-#endif
 }
 
 /*
@@ -1187,6 +1196,7 @@ destroy_Montage(void *obj)
     if (montage->info && montage->info->texture != NULL)
     {
         rm_delete_temp_image(montage->info->texture);
+        magick_free(montage->info->texture);
         montage->info->texture = NULL;
     }
     if (montage->info)
@@ -1248,7 +1258,7 @@ Montage_texture_eq(VALUE self, VALUE texture)
 {
     Montage *montage;
     Image *texture_image;
-    char tmpnam[MaxTextExtent];
+    char temp_name[MaxTextExtent];
 
     Data_Get_Struct(self, Montage, montage);
 
@@ -1256,15 +1266,17 @@ Montage_texture_eq(VALUE self, VALUE texture)
     // remove it now in preparation for this new one.
     if (montage->info->texture)
     {
+        rm_delete_temp_image(montage->info->texture);
         magick_free(montage->info->texture);
         montage->info->texture = NULL;
     }
 
-    Data_Get_Struct(ImageList_cur_image(texture), Image, texture_image);
+    texture = ImageList_cur_image(texture);
+    texture_image = rm_check_destroyed(texture);
 
     // Write a temp copy of the image & save its name.
-    rm_write_temp_image(texture_image, tmpnam);
-    magick_clone_string(&montage->info->texture, tmpnam);
+    rm_write_temp_image(texture_image, temp_name);
+    magick_clone_string(&montage->info->texture, temp_name);
 
     return self;
 }
@@ -1281,7 +1293,7 @@ Montage_tile_eq(VALUE self, VALUE tile_arg)
 
     Data_Get_Struct(self, Montage, montage);
     tile = rb_funcall(tile_arg, rm_ID_to_s, 0);
-    magick_clone_string(&montage->info->tile, STRING_PTR(tile));
+    magick_clone_string(&montage->info->tile, StringValuePtr(tile));
 
     return self;
 }
@@ -1296,7 +1308,7 @@ Montage_title_eq(VALUE self, VALUE title)
     Montage *montage;
 
     Data_Get_Struct(self, Montage, montage);
-    magick_clone_string(&montage->info->title, STRING_PTR(title));
+    magick_clone_string(&montage->info->title, StringValuePtr(title));
     return self;
 }
 
@@ -1329,24 +1341,6 @@ PolaroidOptions_alloc(VALUE class)
 }
 
 
-/*
-    Method:     PolaroidOptions.new
-    Purpose:    Ruby 1.6 singleton function to allocate and init a PolaroidOptions
-                object
-*/
-#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
-VALUE PolaroidOptions_new(VALUE class)
-{
-    volatile VALUE polaroid_obj;
-
-    polaroid_obj = PolaroidOptions_alloc(class);
-    rb_obj_call_init(polaroid_obj, 0, NULL);
-
-    return polaroid_obj;
-}
-#endif
-
-
 /*
     Method:     Magick::PolaroidOptions#initialize
     Purpose:    Yield to an optional block
@@ -1363,6 +1357,7 @@ PolaroidOptions_initialize(VALUE self)
     GetExceptionInfo(&exception);
     (void) QueryColorDatabase("gray75", &draw->shadow_color, &exception);
     CHECK_EXCEPTION()
+    (void) QueryColorDatabase("#dfdfdf", &draw->info->border_color, &exception);
 
     if (rb_block_given_p())
     {
@@ -1380,11 +1375,7 @@ PolaroidOptions_initialize(VALUE self)
 */
 VALUE rm_polaroid_new(void)
 {
-#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
     return PolaroidOptions_initialize(PolaroidOptions_alloc(Class_PolaroidOptions));
-#else
-    return PolaroidOptions_new(Class_PolaroidOptions);
-#endif
 }
 
 
@@ -1402,13 +1393,28 @@ VALUE PolaroidOptions_shadow_color_eq(VALUE self, VALUE shadow)
 }
 
 
+/*
+    Method:     PolaroidOptions#border_color=
+    Purpose:    Set the border color attribute
+*/
+VALUE PolaroidOptions_border_color_eq(VALUE self, VALUE border)
+{
+    Draw *draw;
+
+    Data_Get_Struct(self, Draw, draw);
+    Color_to_PixelPacket(&draw->info->border_color, border);
+    return self;
+}
+
+
 static VALUE get_dummy_tm_img(VALUE klass)
 {
+#define DUMMY_IMG_CLASS_VAR "@@_dummy_img_"
     volatile VALUE dummy_img = 0;
     Info *info;
     Image *image;
 
-    if (rb_cvar_defined(klass, rm_ID__dummy_img_) != Qtrue)
+    if (rb_cvar_defined(klass, rb_intern(DUMMY_IMG_CLASS_VAR)) != Qtrue)
     {
 
         info = CloneImageInfo(NULL);
@@ -1424,10 +1430,9 @@ static VALUE get_dummy_tm_img(VALUE klass)
         (void) DestroyImageInfo(info);
         dummy_img = rm_image_new(image);
 
-        RUBY18(rb_cvar_set(klass, rm_ID__dummy_img_, dummy_img, 0));
-        RUBY16(rb_cvar_set(klass, rm_ID__dummy_img_, dummy_img));
+        rb_cv_set(klass, DUMMY_IMG_CLASS_VAR, dummy_img);
     }
-    dummy_img = rb_cvar_get(klass, rm_ID__dummy_img_);
+    dummy_img = rb_cv_get(klass, DUMMY_IMG_CLASS_VAR);
 
     return dummy_img;
 }
@@ -1440,88 +1445,87 @@ static VALUE get_dummy_tm_img(VALUE klass)
 */
 static VALUE
 get_type_metrics(
-     int argc,
-     VALUE *argv,
-     VALUE self,
-     get_type_metrics_func_t getter)
+                int argc,
+                VALUE *argv,
+                VALUE self,
+                get_type_metrics_func_t getter)
 {
-     static char attrs[] = "OPbcdefghiklmnopqrstuwxyz[@#%";
+    static char attrs[] = "OPbcdefghiklmnopqrstuwxyz[@#%";
  #define ATTRS_L ((int)(sizeof(attrs)-1))
-     Image *image;
-     Draw *draw;
-     TypeMetric metrics;
-     volatile VALUE img;
-     char *text = NULL;
-     long text_l;
-     long x;
-     unsigned int okay;
+    Image *image;
+    Draw *draw;
+    volatile VALUE t;
+    TypeMetric metrics;
+    char *text = NULL;
+    long text_l;
+    long x;
+    unsigned int okay;
 
-     switch (argc)
-     {
-         case 1:                   // use default image
-             text = STRING_PTR_LEN(argv[0], text_l);
+    switch (argc)
+    {
+        case 1:                   // use default image
+            text = rm_str2cstr(argv[0], &text_l);
 
-             for (x = 0; x < text_l; x++)
-             {
-                 // Ensure text string doesn't refer to image attributes.
-                 if (text[x] == '%' && x < text_l-1)
-                 {
-                     int y;
-                     char spec = text[x+1];
+            for (x = 0; x < text_l; x++)
+            {
+                // Ensure text string doesn't refer to image attributes.
+                if (text[x] == '%' && x < text_l-1)
+                {
+                    int y;
+                    char spec = text[x+1];
 
-                     for (y = 0; y < ATTRS_L; y++)
-                     {
-                         if (spec == attrs[y])
-                         {
-                             rb_raise(rb_eArgError,
-                                 "text string contains image attribute reference `%%%c'",
-                                 spec);
-                         }
-                     }
-                 }
-             }
+                    for (y = 0; y < ATTRS_L; y++)
+                    {
+                        if (spec == attrs[y])
+                        {
+                            rb_raise(rb_eArgError,
+                                     "text string contains image attribute reference `%%%c'",
+                                     spec);
+                        }
+                    }
+                }
+            }
 
-             img = get_dummy_tm_img(CLASS_OF(self));
-             Data_Get_Struct(img, Image, image);
-             break;
-         case 2:
-             img = ImageList_cur_image(argv[0]);
-             Data_Get_Struct(img, Image, image);
-             text = STRING_PTR_LEN(argv[1], text_l);
-             break;                  // okay
-         default:
-             rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
-             break;
-     }
+            Data_Get_Struct(get_dummy_tm_img(CLASS_OF(self)), Image, image);
+            break;
+        case 2:
+            t = ImageList_cur_image(argv[0]);
+            image = rm_check_destroyed(t);
+            text = rm_str2cstr(argv[1], &text_l);
+            break;                  // okay
+        default:
+            rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+            break;
+    }
 
-     if (text_l == 0)
-     {
-         rb_raise(rb_eArgError, "no text to measure");
-     }
+    if (text_l == 0)
+    {
+        rb_raise(rb_eArgError, "no text to measure");
+    }
 
-     Data_Get_Struct(self, Draw, draw);
-#if defined(HAVE_INTERPRETIMAGEATTRIBUTES)
-     draw->info->text = InterpretImageAttributes(NULL, image, text);
+    Data_Get_Struct(self, Draw, draw);
+#if defined(HAVE_INTERPRETIMAGEPROPERTIES)
+    draw->info->text = InterpretImageProperties(NULL, image, text);
 #else
-     draw->info->text = TranslateText(NULL, image, text);
+    draw->info->text = InterpretImageAttributes(NULL, image, text);
 #endif
-     if (!draw->info->text)
-     {
-         rb_raise(rb_eArgError, "no text to measure");
-     }
+    if (!draw->info->text)
+    {
+        rb_raise(rb_eArgError, "no text to measure");
+    }
 
-     okay = (*getter)(image, draw->info, &metrics);
+    okay = (*getter)(image, draw->info, &metrics);
 
-     magick_free(draw->info->text);
-     draw->info->text = NULL;
+    magick_free(draw->info->text);
+    draw->info->text = NULL;
 
-     if (!okay)
-     {
-         rm_check_image_exception(image, RetainOnError);
+    if (!okay)
+    {
+        rm_check_image_exception(image, RetainOnError);
 
-         // Shouldn't get here...
-         rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
-                  "Is the FreeType library installed?");
-     }
-     return TypeMetric_from_TypeMetric(&metrics);
+        // Shouldn't get here...
+        rb_raise(rb_eRuntimeError, "Can't measure text. Are the fonts installed? "
+                 "Is the FreeType library installed?");
+    }
+    return TypeMetric_from_TypeMetric(&metrics);
 }
